--[[

	"Terminator" Intro for KEENGINE
        -------------------------------

This file serves two purposes:

 1) It demonstrates how to use the Lua Interpreter for KEENGINE.

 2) It allows modders to create their own intro animation without having to
    recompile the engine itself.

Lua scripts interpreted by KEENGINE can access the following global objects:

ResMan     -- KEENGINE's Resource Manager (allows you to load images & sounds)
Renderer   -- KEENGINE's Renderer (allows you to draw images)
NoiseMaker -- KEENGINE's Sound Engine (allows you to play sounds)
Config     -- KEENGINE's global settings (stuff read from 'game.ini')
Blitz      -- wrapper for user input etc. (only GotInput() right now)

Please note that while syntax errors in a Lua script will simply prevent you
from running them, all Null-Pointer exceptions will cause the game to crash!
So you should carefully test your scripts before you release your mod.

By the way, line comments in Lua script start with '--'. Block comments start
with that thing up in the first line and end with the thing below.

--]]
-------------------------------------------------------------------------------


-- Intro Settings - they define the behavior of the 'shrinking' at the end
local ZOOMSTEPS  =    60
local TitleKx    =   223	-- x coord of "Keen" in title screen
local TitleKy    =     6	-- y coord of "Keen" in title screen
local TitleCx    =    12	-- x coord of "Commander" in title screen
local TitleCy    =     6	-- y coord of "Commander" in title screen
local TitleScale = 0.123	-- scaling factor of "Commander Keen" in title screen (in relation to scroller images)
-- These settings are customized for the 'TechDemo' title screen.
-- Feel free to change them to fit your title screen.


-- Image Variables:
local img_Commander
local img_Keen
local img_Title
local img_Team = {}

-- Other Variables:
local Done
local ticks
local TeamPics

local Cx, Cy
local Kx, Ky
local CKscale
local Tx = {}
local Ty = {}
local Mid = {}
local Top = {}
local introstate, holdtimer
local OldCx, OldCy, OldKx, OldKy

local function NullToNil(object)
	-- This converts a BlitzMax 'Null' object to a Lua 'nil' object to avoid
	-- crashes. The Lua Interpreter will just stop when a nil object is used,
	-- allowing the game to continue with the non-Lua instructions.
	--
	-- Note: This should only be applied to the result of a ResMan.Get* as shown
	-- in the LoadImages() function below.

	if object and Blitz.IsNull(object) == 0 then
		return object
	else
		return nil
	end
end

local function LoadImages()
	img_Commander	= NullToNil(ResMan.GetImage("Graphics/Intro/intro_c.png"))
	img_Keen	= NullToNil(ResMan.GetImage("Graphics/Intro/intro_k.png"))
	img_Team[0]	= NullToNil(ResMan.GetImage("Graphics/Intro/intro_team1.png"))
	img_Team[1]	= NullToNil(ResMan.GetImage("Graphics/Intro/intro_team2.png"))
	img_Team[2]	= NullToNil(ResMan.GetImage("Graphics/Intro/intro_team3.png"))
	img_Team[3]	= NullToNil(ResMan.GetImage("Graphics/Intro/intro_team4.png"))
	img_Title	= NullToNil(ResMan.GetImage(Config.GetTitleImageName()))
	TeamPics = 4
	-- Note: you shouldn't use more than 4 Team Images
end

local function InitIntro()
	introstate = 0
	holdtimer = 0
	ticks = 0
	Done = false
	Cx = Config.GR_WIDTH
	Kx = -(img_Keen.width+40) -- this would crash the game if 'img_Keen' is a Null-Pointer!
	                          -- if 'img_Keen' is 'nil', this will only stop the execution of this Lua script
	Cy = 20
	Ky = 20
	CKscale = 1.0
	
	local i = 0
	while i < TeamPics do
		Tx[i] = (Config.GR_WIDTH-img_Team[i].width)/2
		Ty[i] = 8+Config.GR_HEIGHT
		Mid[i] = ((Config.GR_HEIGHT-img_Team[i].height)/2)
		Top[i] = -img_Team[i].height-8
		i=i+1
	end
end

local function UpdateIntro()
	ticks = ticks+1
	if introstate < TeamPics then
		Cx = Cx-1
		Kx = Kx+(ticks % 2)
		if Ty[introstate] < Mid[introstate] then
			holdtimer = 102*2
			Mid[introstate] = Top[introstate]-42 --make sure we won't hold again
		end
		if holdtimer == 0 then
			Ty[introstate] = Ty[introstate]-2
		else
			holdtimer = holdtimer-1
		end
		if Ty[introstate] < Top[introstate] then
			introstate = introstate+1
		end
	else
		if introstate == TeamPics then
			Cx = Cx-1
			Kx = Kx+(ticks % 2)
			if Kx >= 220 then
				introstate = TeamPics+1
				holdtimer = 0
				OldKx = Kx
				OldKy = Ky
				OldCx = Cx
				OldCy = Cy
			end
		else
			if holdtimer < ZOOMSTEPS then
				holdtimer = holdtimer+1
				Kx = ((ZOOMSTEPS-holdtimer)*OldKx+holdtimer*TitleKx)/ZOOMSTEPS
				Ky = ((ZOOMSTEPS-holdtimer)*OldKy+holdtimer*TitleKy)/ZOOMSTEPS
				Cx = ((ZOOMSTEPS-holdtimer)*OldCx+holdtimer*TitleCx)/ZOOMSTEPS
				Cy = ((ZOOMSTEPS-holdtimer)*OldCy+holdtimer*TitleCy)/ZOOMSTEPS
				CKscale = ((ZOOMSTEPS-holdtimer)*1.0+holdtimer*TitleScale)/ZOOMSTEPS
			else
				Done = true
			end
		end
	end
end

local function DrawIntro()
	Renderer.Clear()
	Renderer.SetColorValue(Config.INTRO_TEXTCOLOR)
	Renderer.SetLightBlend()
	Renderer.SetScal(CKscale, CKscale)
	if img_Commander then Renderer.DrawImg(img_Commander, Cx, Cy) end
	if img_Keen      then Renderer.DrawImg(img_Keen,      Kx, Ky) end
	Renderer.SetScal(1, 1)
	Renderer.SetDefaultBlend()
	Renderer.SetColorValue(-1)
	
	local i = 0
	while i < TeamPics do
		if img_Team[i] then Renderer.DrawImg(img_Team[i], Tx[i], Ty[i]) end
		i=i+1
	end
end

-------------------------------------------------------------------------------
-- If your script needs to return a value to BlitzMax, it must do so via a 
-- 'main' function. The value will be returned as a String, so you cannot 
-- return objects! Only Strings and Numbers (Integer or Float) work.
--
-- Note: 'false' and 'true' are Lua objects and will be returned to BlitzMax as
-- an empty string! The BlitzMax code will not be able to tell if the value was
-- true or false!
-------------------------------------------------------------------------------

function BoolToInt(value)
	if value then
		return 1
	else
		return 0
	end
end

function IntToBool(value)
	return not (value == 0)
end

function main()
	local Abort = false
	LoadImages()
	InitIntro()
	while not Done do
		UpdateIntro()
		DrawIntro()
		Renderer.Refresh()
		if IntToBool(Blitz.GotInput()) then
			Abort = true
			break
		end
	end

	if not Abort then
		-- Fade into Title Screen:
		DrawIntro()
		Abort = IntToBool(Renderer.FizzleFadeDown(img_Title))
	end
	
	NoiseMaker.StopAll()	-- make sure you stop all sounds when your Intro finishes!
	
	return BoolToInt(Abort)	-- returns the string "0" or "1" to BlitzMax
end
